import 'package:either_dart/either.dart';
import 'package:finbox_dc_plugin/finbox_dc_plugin.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'constants.dart';
import 'common.dart';

void main() {
  runApp(MyApp(key: null,));
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  final Future<FirebaseApp> _initialization = Firebase.initializeApp();
  late FirebaseMessaging _messaging;

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: _initialization,
        builder: (context, snapshot) {
          // Check for errors
          if (snapshot.hasError) {
            // return SomethingWentWrong();
            print('Something went wrong in Flutter Fire');
          }

          if (snapshot.connectionState == ConnectionState.done) {
            print('firebase INIT');
            _messaging = FirebaseMessaging.instance;
            _messaging.getToken().then((value) => print("Fcm token: $value"));
            FirebaseMessaging.onMessage.listen((RemoteMessage event) {
              print("Message Received: ${event.data}");
              FinBoxDcPlugin.forwardFinBoxNotificationToSDK(event.data);
            });
            return MaterialApp(
              title: 'Flutter Demo',
              theme: ThemeData(
                primarySwatch: Colors.blue,
              ),
              home: HomePage(),
            );
          }

          return Container(
            color: Colors.white,
          );
        });
  }
}

class HomePage extends StatelessWidget {

  _createUser() {
    try {
      final hello = FinBoxDcPlugin.createUser(
          Constants.CLIENT_API_KEY, CommonUitl.getUsername());
      hello.fold(
          (left) => {
                // error response
                print("Error: $left")
              },
          (right) => {
                // success response
                print("Access Token: $right")
              });
    } on PlatformException catch (e) {
      print('Failed to fetch data: ${e.message}');
    }
  }

  _startPeriodicSync() {
    FinBoxDcPlugin.startPeriodicSync();
  }

  _stopPeriodicSync() {
    FinBoxDcPlugin.stopPeriodicSync();
  }

  _setSyncFrequency() {
    FinBoxDcPlugin.setSyncFrequency(36000);
  }

  _resetData() {
    FinBoxDcPlugin.resetData();
  }

  _setDeviceMatch() {
    FinBoxDcPlugin.setDeviceMatch("email", "userName", "phone");
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Device Connect Sample"),
      ),
      body: Center(
        child: Column(
          children: [
            ElevatedButton(
              onPressed: _createUser,
              child: Text("Create User"),
            ),
            ElevatedButton(
              onPressed: _setDeviceMatch,
              child: Text("Set Device Data"),
            ),
            ElevatedButton(
              onPressed: _setSyncFrequency,
              child: Text("Set Sync Frequency"),
            ),
            ElevatedButton(
              onPressed: _startPeriodicSync,
              child: Text("Start Periodic Sync"),
            ),
            ElevatedButton(
              onPressed: _stopPeriodicSync,
              child: Text("Stop Periodic Sync"),
            ),
            ElevatedButton(
              onPressed: _resetData,
              child: Text("Reset Data"),
            ),
          ],
        ),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}
